Package: profileModel
Type: Package
Title: Profiling Inference Functions for Various Model Classes
Version: 0.6.2
Authors@R: person(given = "Ioannis", family = "Kosmidis", role = c("aut", "cre"), email = "ioannis.kosmidis@warwick.ac.uk", comment = c(ORCID = "0000-0003-1556-0302"))
URL: https://github.com/ikosmidis/profileModel
BugReports: https://github.com/ikosmidis/profileModel/issues
Description: Provides tools that can be used to calculate, evaluate, plot and use for inference the profiles of *arbitrary* inference functions for *arbitrary* 'glm'-like fitted models with linear predictors. More information on the methods that are implemented can be found in Kosmidis (2008) <https://www.r-project.org/doc/Rnews/Rnews_2008-2.pdf>.
License: GPL (>= 2)
Depends: R (>= 2.6.0)
Suggests: MASS, gnm
RoxygenNote: 7.1.1
NeedsCompilation: no
Packaged: 2026-02-06 16:30:04 UTC; yiannis
Author: Ioannis Kosmidis [aut, cre] (ORCID:
    <https://orcid.org/0000-0003-1556-0302>)
Maintainer: Ioannis Kosmidis <ioannis.kosmidis@warwick.ac.uk>
Repository: CRAN
Date/Publication: 2026-02-06 17:00:02 UTC
Built: R 4.6.0; ; 2026-04-25 16:22:27 UTC; unix
