export(confintModel, ordinaryDeviance,
       RaoScoreStatistic, plot.profileModel, pairs.profileModel,
       print.profileModel, profileModel,
       profConfint, profZoom, profSmooth,
       profConfint.profileModel, profZoom.profileModel,
       profSmooth.profileModel,
       profiling, prelim.profiling,
       signedSquareRoots.profileModel)

S3method(profConfint,profileModel)
S3method(profZoom,profileModel)
S3method(profSmooth,profileModel)
S3method(plot,profileModel)
S3method(print,profileModel)
S3method(pairs, profileModel)
S3method(signedSquareRoots, profileModel)

importFrom("graphics", "axis", "frame", "lines", "mtext", "par",
           "plot", "points", "segments", "text")
importFrom("stats", "approx", "as.formula", "coef", "fitted",
           "formula", "model.frame", "model.matrix", "model.offset",
           "model.response", "qchisq", "spline", "update")
